# Copyright 2021 The LeafNLP Team. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import torch

class natsEmbedding(torch.nn.Module):

    def __init__(self,
                 vocab_size, emb_dim,
                 share_emb_weight=True):
        """
        embedding and decoding.
        """
        super().__init__()

        self.embedding = torch.nn.Embedding(vocab_size, emb_dim)
        torch.nn.init.uniform_(self.embedding.weight, -1.0, 1.0)

        """
        For embedding weight sharing.
        """
        if share_emb_weight:
            self.proj2vocab = torch.nn.Linear(
                emb_dim, vocab_size)
            self.proj2vocab.weight.data = self.embedding.weight.data

    def forward(self, input_):
        """
        get the embedding
        """
        return self.embedding(input_)

    def get_embedding(self, input_):
        """
        get the embedding
        """
        return self.embedding(input_)

    def get_decode2vocab(self, input_):
        """
        from vector to vocab prediction
        """
        return self.proj2vocab(input_)
