# Copyright 2021 The LeafNLP Team. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse

from leafnlp.LeafBase.utils.utils import str2bool

parser = argparse.ArgumentParser()
"""
Use in the framework and cannot remove.
"""
parser.add_argument('--debug', type=str2bool, default=False, 
                    help='Debug?')
parser.add_argument('--task', default='train',
                    help='train | evaluate')

parser.add_argument('--data_dir', default='../../data', 
                    help='data dir')
parser.add_argument('--work_dir', default='../../work_dir', 
                    help='work dir')
parser.add_argument('--file_train', default='train.jsonl', 
                    help='train data.')
parser.add_argument('--file_val', default='dev.jsonl', 
                    help='dev data')
parser.add_argument('--file_test', default='test.jsonl', 
                    help='test data')
parser.add_argument('--file_output', default='output.jsonl', 
                    help='output data')

parser.add_argument('--base_model_dir', default='../base_model_dir', 
                    help='base model dir')
parser.add_argument('--train_model_dir', default='../train_model_dir', 
                    help='train model dir')

parser.add_argument('--n_epoch', type=int, default=10,
                    help='number of epochs.')
parser.add_argument('--batch_size', type=int, default=20, 
                    help='batch size.')
parser.add_argument('--checkpoint', type=int, default=500,
                    help='How often you want to save model?')
parser.add_argument('--val_n_batches', type=int, default=400, 
                    help='how many batches validation')
parser.add_argument('--n_best_models', type=int, default=3,
                    help='How many models you want to keep?')
parser.add_argument('--validate_once', type=str2bool, default=True,
                    help='Only validate once.')
parser.add_argument('--continue_training', type=str2bool, default=True, 
                    help='Do you want to continue?')
parser.add_argument('--train_base_model', type=str2bool, default=False,
                    help='True: Use Pretrained Param | False: Transfer Learning')
parser.add_argument('--is_lower', type=str2bool, default=False,
                    help='lower case for all words?')
parser.add_argument('--use_move_avg', type=str2bool, default=False, 
                    help='move average')
parser.add_argument('--use_optimal_model', type=str2bool, default=True, 
                    help='Do you want to use the best model?')
parser.add_argument('--model_optimal_key', default='0,0', 
                    help='epoch,batch')
"""
User specified parameters.
"""
parser.add_argument('--device', default="cuda:0", help='device')
# learning rate and scheduler
# Schedule learning rate. 
parser.add_argument('--lr_schedule', default='None',
                    help='decay | warm-up | None')
parser.add_argument('--learning_rate', type=float, default=0.0005,
                    help='learning rate.')
parser.add_argument('--grad_clip', type=float, default=1.0,
                    help='clip the gradient norm.')
parser.add_argument('--step_size', type=int, default=1,
                    help='step size')
parser.add_argument('--step_decay', type=float, default=0.5,
                    help='learning rate decay')
parser.add_argument('--warmup_step', type=int, default=2000,
                    help='warmup step size')
parser.add_argument('--model_size', type=int, default=100000,
                    help='model size')
# dropout
parser.add_argument('--drop_rate', type=float, default=0.2, 
                    help='dropout.')

args = parser.parse_args()

"""
run model
"""
import torch
args.device = torch.device(args.device)
from .trainer import trainerSeqLabel
model = trainerSeqLabel(args)
if args.task == 'train':
    model.train()
if args.task == 'validate':
    model.validate()
if args.task == 'test':
    model.test()
