# Copyright 2021 The LeafNLP Team. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import torch
from leafnlp.LeafBase.engines.end2end_large import End2EndBase


class PipeSeqLabelBase(End2EndBase):
    """
    FOR Sequence Labeling.
    """

    def __init__(self, args):
        super().__init__(args=args)

        self.loss_criterion = torch.nn.CrossEntropyLoss(
            ignore_index=-1).to(self.args.device)

    def build_scheduler(self, optimizer):
        """
        Schedule Learning Rate
        """
        if self.args.lr_schedule:
            scheduler = torch.optim.lr_scheduler.StepLR(
                optimizer=optimizer, step_size=self.args.step_size,
                gamma=self.args.step_decay)

        return scheduler

    def build_optimizer(self, params):
        """
        init model optimizer
        """
        optimizer = torch.optim.Adam(params, lr=self.args.learning_rate)

        return optimizer
